import time

from django.apps import apps
from django.core.cache import cache
from django.dispatch import receiver
from utg_base.permissions.utils import sync_permissions
from utg_base.utils.signals import app_ready


@receiver(app_ready)
def sync_perms(sender, **kwargs):
    # wait all apps ready
    while not apps.ready:
        time.sleep(0.1)

    # only run 1 time (not all gunicorn workers)
    if not cache.add("sync_perms", True, timeout=10):
        return
    sync_permissions()



import os

from django.core.management.commands.makemigrations import Command as BaseMakeMigrations
from django.core.management.utils import run_formatters
from django.db.migrations.writer import MigrationWriter
from django.db.migrations.operations import CreateModel


class Command(BaseMakeMigrations):

    def write_migration_files(self, changes, update_previous_migration_paths=None):
        """
        Take a changes dict and write them out as migration files.
        """
        directory_created = {}
        for app_label, app_migrations in changes.items():
            if self.verbosity >= 1:
                self.log(self.style.MIGRATE_HEADING("Migrations for '%s':" % app_label))
            for migration in app_migrations:
                # Describe the migration
                writer = MigrationWriter(migration, self.include_header)

                # BEGIN Customizing
                # fix 'created_at', 'updated_at' fields order
                for operation in migration.operations:
                    if not isinstance(operation, CreateModel):
                        continue
                    _fields = []
                    _end_fields = []
                    for field_name, field in operation.fields:
                        if field_name in ['created_at', 'updated_at']:
                            _end_fields.append((field_name, field))
                        else:
                            _fields.append((field_name, field))

                    operation.fields = _fields + _end_fields

                # END Customizing

                if self.verbosity >= 1:
                    # Display a relative path if it's below the current working
                    # directory, or an absolute path otherwise.
                    migration_string = self.get_relative_path(writer.path)
                    self.log("  %s\n" % self.style.MIGRATE_LABEL(migration_string))
                    for operation in migration.operations:
                        self.log("    %s" % operation.formatted_description())
                    if self.scriptable:
                        self.stdout.write(migration_string)
                if not self.dry_run:
                    # Write the migrations file to the disk.
                    migrations_directory = os.path.dirname(writer.path)
                    if not directory_created.get(app_label):
                        os.makedirs(migrations_directory, exist_ok=True)
                        init_path = os.path.join(migrations_directory, "__init__.py")
                        if not os.path.isfile(init_path):
                            open(init_path, "w").close()
                        # We just do this once per app
                        directory_created[app_label] = True
                    migration_string = writer.as_string()
                    with open(writer.path, "w", encoding="utf-8") as fh:
                        fh.write(migration_string)
                        self.written_files.append(writer.path)
                    if update_previous_migration_paths:
                        prev_path = update_previous_migration_paths[app_label]
                        rel_prev_path = self.get_relative_path(prev_path)
                        if writer.needs_manual_porting:
                            migration_path = self.get_relative_path(writer.path)
                            self.log(
                                self.style.WARNING(
                                    f"Updated migration {migration_path} requires "
                                    f"manual porting.\n"
                                    f"Previous migration {rel_prev_path} was kept and "
                                    f"must be deleted after porting functions manually."
                                )
                            )
                        else:
                            os.remove(prev_path)
                            self.log(f"Deleted {rel_prev_path}")
                elif self.verbosity == 3:
                    # Alternatively, makemigrations --dry-run --verbosity 3
                    # will log the migrations rather than saving the file to
                    # the disk.
                    self.log(
                        self.style.MIGRATE_HEADING(
                            "Full migrations file '%s':" % writer.filename
                        )
                    )
                    self.log(writer.as_string())
        run_formatters(self.written_files, stderr=self.stderr)

    def handle(self, *args, **options):
        self.written_files = []
        result = super().handle(*args, **options)
        return result



from rest_framework import serializers
from rest_framework.exceptions import ValidationError

from utg_base.utils.date import to_udate, to_udatetime
from utg_base.utils.translation import translate as _


class UDateField(serializers.DateField):
    def to_internal_value(self, value):
        return to_udate(super().to_internal_value(value))


class UDateTimeField(serializers.DateTimeField):
    def to_internal_value(self, value):
        return to_udatetime(super().to_internal_value(value))


class DateSerializer(serializers.Serializer):
    dt = UDateField()


class DatePeriodSerializer(DateSerializer):
    period = serializers.ChoiceField(choices=['daily', 'monthly', 'yearly'], default='daily')


class DateTimePeriodSerializer(DateSerializer):
    dt = UDateTimeField(required=False)
    period = serializers.ChoiceField(choices=['hourly', 'today', 'daily', 'monthly', 'yearly'], default='daily')

    def validate(self, attrs):
        period = attrs.get('period', 'daily')
        dt = attrs.get('dt')

        if period != 'today' and dt is None:
            raise ValidationError({
                'dt': _('This field is required when period is not "today".')
            })
        return attrs


class RelatedObjectField(serializers.PrimaryKeyRelatedField):
    """
    POST / PUT  -> UUID (string)
    RESPONSE    -> {id, name}
    """
    fields = []

    def __init__(self, **kwargs):
        self.fields = kwargs.pop("fields", ("id", "name"))
        super().__init__(**kwargs)

    def use_pk_only_optimization(self):
        return False

    def to_representation(self, value):
        return {f: str(getattr(value, f)) for f in self.fields}



from typing import Literal

from django_redis import get_redis_connection
from redis.client import Redis


def generate_folder_cache_key(user_id: str):
    return f"user:{user_id}:folders"


def has_folder_perm(user_id, folder_perm):
    redis_conn: Redis = get_redis_connection("shared")
    return bool(redis_conn.sismember(generate_folder_cache_key(user_id), folder_perm))


def has_folder_perms(user_id: str, folder_perms: list[str], operator: Literal["OR", "AND"] = "OR"):
    redis_conn: Redis = get_redis_connection("shared")
    result = redis_conn.smismember(generate_folder_cache_key(user_id), folder_perms)

    if operator == "OR":
        return any(result)
    return all(result)



import logging
import os

from utg_base.env import env


class UtgBaseFilter(logging.Filter):
    BASE_DIR = os.getcwd()

    def filter(self, record):
        sp = '/site-packages/'
        path = record.pathname

        if record.pathname.startswith(str(self.BASE_DIR)):
            path = record.pathname[len(str(self.BASE_DIR)):]
        elif sp in path:
            path = path[path.index(sp, 0) + len(sp):]

        path = path.replace('/', '.')
        path = path.removeprefix('.')
        record.path = path
        return record


class UtgBaseLogging:
    enable_console_info = int(env('LOGGING_CONSOLE_INFO', True))
    enable_db_debug = int(env('LOGGING_DB_DEBUG', False))
    enable_loki = int(env('LOGGING_LOKI', True))
    enable_loki_debug = int(env('LOGGING_LOKI_DEBUG', False))

    logger = logging.getLogger('main')

    format = '[%(asctime)s] %(levelname)s [%(path)s:%(lineno)s] [%(funcName)s] %(message)s'

    __LOGGING = {
        'version': 1,
        'disable_existing_loggers': False,
        'filters': {
            'trim_path': {
                '()': 'utg_base.logging.UtgBaseFilter',
            },
        },
        'formatters': {
            'simple': {
                'format': format,
                'datefmt': '%Y-%m-%d %H:%M:%S',
            },
            'loki': {
                'class': 'django_loki.LokiFormatter',  # required
                'format': format,  # optional, default is logging.BASIC_FORMAT
                'datefmt': '%Y-%m-%d %H:%M:%S',  # optional, default is '%Y-%m-%d %H:%M:%S'
            },
        },
        'handlers': {
            'console_info': {
                'level': 'INFO',
                'class': 'logging.StreamHandler',
                'formatter': 'simple',
                'filters': ['trim_path'],
            },
            'console_debug': {
                'level': 'DEBUG',
                'class': 'logging.StreamHandler',
                'formatter': 'simple',
                'filters': ['trim_path'],
            },
            'loki': {
                'level': env('LOKI_LEVEL', 'ERROR'),  # required
                'class': 'django_loki.LokiHttpHandler',  # required
                'host': env('LOKI_HOST', env('DB_HOST')),
                # required, your grafana/Loki server host, e.g:192.168.25.30
                'formatter': 'loki',  # required, loki formatter,
                'port': int(env('LOKI_PORT', 3100)),
                # optional, your grafana/Loki server port, default is 3100
                'timeout': float(env('LOKI_TIMEOUT', 0.5)),
                # optional, request Loki-server by http or https time out, default is 0.5
                'protocol': env('LOKI_PROTOCOL', 'http'),
                # optional, Loki-server protocol, default is http
                'source': env('LOKI_SOURCE', 'Loki'),  # optional, label name for Loki, default is Loki
                'src_host': env('LOKI_SRC_HOST', 'localhost'),
                # optional, label name for Loki, default is localhost
                'tz': 'Asia/Tashkent',
                # optional, timezone for formatting timestamp, default is UTC, e.g:Asia/Shanghai
                'filters': ['trim_path'],
            },
        },
        'loggers': {
            'django': {
                'handlers': [],
                'level': 'DEBUG',
                'propagate': True,
            },
            'django.db.backends': {
                'handlers': [],
                'level': 'DEBUG',
                'propagate': True,
            },
            'main': {
                'handlers': [],
                'level': 'DEBUG',
                'propagate': True,
            }
        },
    }

    # noinspection PyUnresolvedReferences
    @classmethod
    def logging(cls):
        _logging = cls.__LOGGING.copy()

        if cls.enable_console_info:
            _logging['loggers']['django']['handlers'].append('console_info')
        if cls.enable_db_debug:
            _logging['loggers']['django.db.backends']['handlers'].append('console_debug')
        if cls.enable_loki:
            _logging['loggers']['django']['handlers'].append('loki')
        if cls.enable_loki_debug:
            _logging['handlers']['loki_debug'] = _logging['handlers']['loki'].copy()
            _logging['handlers']['loki_debug']['level'] = 'DEBUG'
            _logging['loggers']['main']['handlers'].append('loki_debug')
            _logging['loggers']['main']['handlers'].append('console_debug')

        return _logging


LOGGING = UtgBaseLogging.logging()
logger = UtgBaseLogging.logger



from django.utils import timezone
from django.utils.translation import activate, get_language
from rest_framework.request import Request
from rest_framework.views import APIView

from utg_base.constants import AVAILABLE_LANGUAGES
from utg_base.models import JWTUser


class BaseRequest(Request):

    @property
    def user(self) -> JWTUser:
        return super(BaseRequest, self).user()


class BaseAPIView(APIView):
    user_id_assign_fields_on_create = []
    user_id_assign_fields_on_update = []
    user_id_assign_fields_on_delete = []
    deleted_at_field = None

    def __init__(self, **kwargs):
        # Check user_id_assign_fields_on_create
        assert isinstance(self.user_id_assign_fields_on_create, list), (
            'Expected iterable user_id_assign_fields_on_create field'
        )

        # Check user_id_assign_fields_on_update
        assert isinstance(self.user_id_assign_fields_on_update, list), (
            'Expected iterable user_id_assign_fields_on_update field'
        )

        # Check user_id_assign_fields_on_delete
        assert isinstance(self.user_id_assign_fields_on_delete, list), (
            'Expected iterable user_id_assign_fields_on_delete field'
        )

        # Check deleted_at_field
        assert isinstance(self.deleted_at_field, (str, type(None))), (
            'Expected str | None deleted_at_field field'
        )
        super().__init__(**kwargs)
        self.request: BaseRequest | None = None

    def perform_create(self, serializer):
        kws = {}
        for attr in getattr(self, 'user_id_assign_fields_on_create', []):
            kws[attr] = self.request.user.id
        serializer.save(**kws)

    def perform_update(self, serializer):
        kws = {}
        for attr in getattr(self, 'user_id_assign_fields_on_update', []):
            kws[attr] = self.request.user.id
        serializer.save(**kws)

    def perform_destroy(self, instance):
        for attr in getattr(self, 'user_id_assign_fields_on_delete', []):
            setattr(instance, attr, self.request.user.id)
        if self.deleted_at_field and hasattr(instance, self.deleted_at_field):
            setattr(instance, self.deleted_at_field, timezone.now())
        instance.save()

    def update_lang(self):
        activate(self.get_language())

    def get_language(self):
        if lang_from_header := self.request.headers.get('accept-language'):
            if lang_from_header and lang_from_header not in AVAILABLE_LANGUAGES:
                lang_from_header = 'ru'

        return lang_from_header or get_language()



# Generated by Django 5.2.8 on 2025-12-02 12:59

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='UService',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('title_translate_key', models.CharField(max_length=255)),
                ('logo', models.FileField(upload_to='media/logo')),
                ('type', models.CharField(choices=[('FILE_UPLOAD', 'File Upload'), ('API_INTEGRATION', 'Api Integration'), ('ADMIN', 'Admin')], max_length=20)),
                ('ip', models.GenericIPAddressField()),
                ('port', models.PositiveSmallIntegerField()),
            ],
            options={
                'ordering': ['name'],
            },
        ),
    ]



from requests import Response


# noinspection PyDecorator
@staticmethod
def call_processor(response: Response):
    try:
        data = response.json()
    except Exception as e:
        data = response.text
    return {
        'status': response.status_code,
        'reason': response.reason,
        'response': data,
    }


# noinspection PyDecorator
@staticmethod
def data_processor(response: Response):
    try:
        data = response.json()
    except Exception as e:
        raise Exception({
            'status': response.status_code,
            'reason': response.reason,
            'response': response.text,
            'exception': str(e)
        })

    return data



import json

from celery import current_app
from django_celery_beat.models import PeriodicTask
from drf_spectacular.utils import extend_schema
from rest_framework import viewsets, filters
from rest_framework.exceptions import NotFound, ValidationError
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView

from utg_base.api.permissions import IsSuperUser
from utg_base.celery.serializers import PeriodicTaskSerializer
from utg_base.utils.translation import translate as _


@extend_schema(tags=['admin/periodic-tasks'])
class PeriodicTaskViewSet(viewsets.ModelViewSet):
    http_method_names = ['get', 'patch']
    queryset = PeriodicTask.objects.all()
    serializer_class = PeriodicTaskSerializer
    permission_classes = [IsSuperUser]
    filter_backends = [filters.SearchFilter]
    search_fields = ['name']


@extend_schema(tags=['admin/periodic-tasks'])
class PeriodicTaskRunNowView(APIView):
    permission_classes = [IsAuthenticated]

    def patch(self, request, pk):
        try:
            periodic_task = PeriodicTask.objects.get(pk=pk)
        except PeriodicTask.DoesNotExist:
            raise NotFound(detail=_("Periodic task not found"))

        if not periodic_task.enabled:
            raise ValidationError(detail=_("Task is disabled"))

        args = json.loads(periodic_task.args) if periodic_task.args else []
        kwargs = json.loads(periodic_task.kwargs) if periodic_task.kwargs else {}

        task = current_app.send_task(
            periodic_task.task,
            args=args,
            kwargs=kwargs,
            countdown=0,
            headers = {'periodic_task_name': periodic_task.name}
        )

        return Response({
            "detail": _("Task successfully triggered"),
            "task_id": task.id,
            "task_name": periodic_task.name,
            "celery_task": periodic_task.task,
        })



from django_celery_results.models import TaskResult
from rest_framework import serializers


class TaskResultSerializer(serializers.ModelSerializer):
    class Meta:
        model = TaskResult
        fields = '__all__'



import sys
import threading

from django.dispatch import Signal

app_ready = Signal()


def register_app_ready_signals():
    if any([arg in sys.argv for arg in ['runserver', 'gunicorn', 'uwsgi', 'core.wsgi', 'core.asgi']]):
        threading.Thread(target=app_ready.send, kwargs={'sender': True}, daemon=True).start()



from .utils import get_model_classes, create_view_set, get_basename, get_url_prefix
from ..api.routers import OptionalSlashRouter

router = OptionalSlashRouter()

for model_class in get_model_classes():
    router.register(
        prefix=get_url_prefix(model_class),
        viewset=create_view_set(model_class),
        basename=get_basename(model_class)
    )

urlpatterns = router.urls



import os
import typing
from functools import wraps

import dotenv
import hvac
from django.conf import settings
from environs import Env, _StrType, EnvError

dotenv.load_dotenv(settings.BASE_DIR / '.env')

assert os.environ.get('VAULT_URL') is not None, "VAULT_URL not defined, please set VAULT_URL on .env file"
assert os.environ.get('VAULT_TOKEN') is not None, "VAULT_TOKEN not defined, please set VAULT_TOKEN on .env file"
assert os.environ.get('VAULT_PATH') is not None, "VAULT_PATH not defined, please set VAULT_PATH on .env file"

client = hvac.Client(
    url=os.environ.get("VAULT_URL"),
    token=os.environ.get("VAULT_TOKEN"),
    verify=False
)
if not client.is_authenticated():
    raise Exception("Vault authentication failed")

envs = client.secrets.kv.read_secret_version(
    path=os.environ.get("VAULT_PATH"),
    mount_point="utg-scada"
)['data']['data']


class VaultEnv(Env):
    _wrap_methods = {
        'int', 'bool', 'str', 'float', 'decimal',
        'list', 'dict', 'json',
        'datetime', 'date', 'time', 'timedelta',
        'path', 'log_level', 'uuid', 'url', 'enum',
        'dj_db_url', 'dj_email_url', 'dj_cache_url'
    }

    def __call__(self, key, default=None, **kwargs):
        try:
            return super().__call__(key, default=default, **kwargs)
        except EnvError:
            return default

    def __getattribute__(self, name):
        attr = super().__getattribute__(name)

        if callable(attr) and name in self._wrap_methods:
            @wraps(attr)
            def wrapper(*args, **kwargs):
                if len(args) < 2:
                    kwargs.setdefault("default", None)
                return attr(*args, **kwargs)

            return wrapper

        return attr

    def _get_value(self, env_key: _StrType, default: typing.Any) -> typing.Any:
        if env_key in os.environ:
            return os.environ.get(env_key, default)
        else:
            return envs.get(env_key, default)


env = VaultEnv()
